@@ -310,8 +310,8 @@ struct ClaspFacade::SolveData {
 			}
 			algo->setEnumLimit(numM ? static_cast<uint64>(numM) : UINT64_MAX);
 			costs.data = lastModel();
-			prepared = true;
 		}
+		prepared = true;
 	}
 	bool update(const Solver& s, const Model& m) { return !active->handler || active->handler->onModel(s, m); }
 	bool interrupt(int sig) {
@@ -542,6 +542,7 @@ struct SummaryStats {
 };
 
 double _getConcurrency(const SharedContext* ctx) { return ctx->concurrency(); }
+double _getWinner(const SharedContext* ctx) { return ctx->winner(); }
 double _getResult(const SolveResult* r) { return static_cast<double>(r->operator Clasp::SolveResult::Base()); }
 double _getSignal(const SolveResult* r) { return static_cast<double>(r->signal); }
 double _getExhausted(const SolveResult* r) { return static_cast<double>(r->exhausted()); }
@@ -676,6 +677,7 @@ ClaspFacade::Statistics::ClingoView::ClingoView(const ClaspFacade& f) {
 	summary_.add("exhausted"  , StatisticObject::value<SolveResult, _getExhausted>(&f.step_.result));
 	summary_.add("costs"      , StatisticObject::array(&f.solve_->costs));
 	summary_.add("concurrency", StatisticObject::value<SharedContext, _getConcurrency>(&f.ctx));
+	summary_.add("winner"     , StatisticObject::value<SharedContext, _getWinner>(&f.ctx));
 	summary_.step.bind(f.step_);
 	summary_.step.addTo(summary_);
 	if (f.step_.lpStats()) {
@@ -923,7 +925,7 @@ void ClaspFacade::prepare(EnumMode enumMode) {
 		prg->getWeakBounds(en.optBound);
 	}
 	stats_->start(uint32(config_->context().stats));
-	if (en.optMode != MinimizeMode_t::ignore && (m = ctx.minimize()) != 0) {
+	if (ctx.ok() && en.optMode != MinimizeMode_t::ignore && (m = ctx.minimize()) != 0) {
 		if (!m->setMode(en.optMode, en.optBound)) {
 			assume_.push_back(lit_false());
 		}
