@@ -223,6 +223,10 @@ void LogicProgram::dispose(bool force) {
 		delete incData_;
 		VarVec().swap(propQ_);
 		stats.reset();
+		incData_ = 0;
+		theory_  = 0;
+		input_   = AtomRange(1, UINT32_MAX);
+		statsId_ = 0;
 	}
 	rule_.clear();
 }
@@ -240,10 +244,7 @@ bool LogicProgram::doStartProgram() {
 	trueAt->setInUpper(true);
 	trueAt->setLiteral(lit_true());
 	atomState_.set(0, AtomState::fact_flag);
-	incData_ = 0;
 	auxData_ = new Aux();
-	input_   = AtomRange(1, UINT32_MAX);
-	statsId_ = 0;
 	return true;
 }
 void LogicProgram::setOptions(const AspOptions& opts) {
@@ -1052,6 +1053,11 @@ void LogicProgram::updateFrozenAtoms() {
 
 void LogicProgram::prepareProgram(bool checkSccs) {
 	assert(!frozen());
+	prepareExternals();
+	// Given that freezeTheory() might introduce otherwise
+	// unused atoms, it must be called before we fix the
+	// number of input atoms.
+	freezeTheory();
 	uint32 nAtoms = (input_.hi = std::min(input_.hi, endAtom()));
 	stats.auxAtoms += endAtom() - nAtoms;
 	for (uint32 i = 0; i != RuleStats::numKeys(); ++i) {
@@ -1062,8 +1068,6 @@ void LogicProgram::prepareProgram(bool checkSccs) {
 	}
 	statsId_ = 1;
 	transformExtended();
-	prepareExternals();
-	freezeTheory();
 	updateFrozenAtoms();
 	PrgAtom* suppAtom = 0;
 	if (opts_.suppMod) {
@@ -1144,15 +1148,14 @@ bool LogicProgram::TFilter::operator()(const Potassco::TheoryAtom& a) const {
 	return !at->frozen();
 }
 struct LogicProgram::DlpTr : public RuleTransform::ProgramAdapter {
-	DlpTr(LogicProgram* x, EdgeType et) : self(x), type(et), scc(0) {}
+	DlpTr(LogicProgram* x, EdgeType et) : self(x), type(et), scc(PrgNode::noScc) {}
 	Atom_t newAtom() {
-		CLASP_FAIL_IF(type == PrgEdge::Gamma, "shifting must not introduce new atoms!");
 		Atom_t x   = self->newAtom();
 		PrgAtom* a = self->getAtom(x);
-		self->auxData_->scc.push_back(a);
 		a->setScc(scc);
 		a->setSeen(true);
 		atoms.push_back(x);
+		if (scc != PrgNode::noScc) { self->auxData_->scc.push_back(a); }
 		return x;
 	}
 	virtual void addRule(const Rule& r) {
@@ -1167,6 +1170,18 @@ struct LogicProgram::DlpTr : public RuleTransform::ProgramAdapter {
 			self->stats.gammas += uint32(gamma);
 		}
 	}
+	void assignAuxAtoms() {
+		self->incTrAux(sizeVec(atoms));
+		while (!atoms.empty()) {
+			PrgAtom* ax = self->getAtom(atoms.back());
+			atoms.pop_back();
+			if (ax->supports()) {
+				ax->setInUpper(true);
+				ax->assignVar(*self, *ax->supps_begin());
+			}
+			else { self->assignValue(ax, value_false, PrgEdge::noEdge()); }
+		}
+	}
 	LogicProgram* self;
 	EdgeType      type;
 	uint32        scc;
@@ -1264,7 +1279,8 @@ void LogicProgram::finalizeDisjunctions(Preprocessor& p, uint32 numSccs) {
 						nonHcfs_.add(scc);
 					}
 					if (!options().noGamma) {
-						shifter.transform(sr);
+						tr.scc = scc;
+						shifter.transform(sr, RuleTransform::strategy_select_no_aux);
 					}
 					else {
 						// only add support edge
@@ -1276,6 +1292,7 @@ void LogicProgram::finalizeDisjunctions(Preprocessor& p, uint32 numSccs) {
 			}
 		}
 	}
+	tr.assignAuxAtoms();
 	if (!disjunctions_.empty() && nonHcfs_.config == 0) {
 		nonHcfs_.config = ctx()->configuration()->config("tester");
 	}
@@ -1335,16 +1352,7 @@ void LogicProgram::prepareComponents() {
 				}
 			}
 		}
-		incTrAux(sizeVec(tr.atoms));
-		while (!tr.atoms.empty()) {
-			PrgAtom* ax = getAtom(tr.atoms.back());
-			tr.atoms.pop_back();
-			if (ax->supports()) {
-				ax->setInUpper(true);
-				ax->assignVar(*this, *ax->supps_begin());
-			}
-			else { assignValue(ax, value_false, PrgEdge::noEdge()); }
-		}
+		tr.assignAuxAtoms();
 		setFrozen(true);
 	}
 }
@@ -1709,11 +1717,15 @@ bool LogicProgram::simplifyRule(const Rule& r, Potassco::RuleBuilder& out, SRule
 // create new atom aux representing supports, i.e.
 // aux == S1 v ... v Sn
 Literal LogicProgram::getEqAtomLit(Literal lit, const BodyList& supports, Preprocessor& p, const SccMap& sccMap) {
-	if (supports.empty() || lit == lit_false()) { return lit_false(); }
-	if (supports.size() == 1 && supports[0]->size() < 2) {
+	if (supports.empty() || lit == lit_false()) {
+		return lit_false();
+	}
+	else if (supports.size() == 1 && supports[0]->size() < 2) {
 		return supports[0]->size() == 0 ? lit_true() : supports[0]->goal(0);
 	}
-	if (p.getRootAtom(lit) != varMax)         { return posLit(p.getRootAtom(lit)); }
+	else if (p.getRootAtom(lit) != varMax) {
+		return posLit(p.getRootAtom(lit));
+	}
 	incTrAux(1);
 	Atom_t auxV  = newAtom();
 	PrgAtom* aux = getAtom(auxV);
@@ -1729,7 +1741,9 @@ Literal LogicProgram::getEqAtomLit(Literal lit, const BodyList& supports, Prepro
 				if (aScc != PrgNode::noScc && (sccMap[aScc] & 1u)) { scc = aScc; }
 			}
 			b->addHead(aux, PrgEdge::Normal);
-			if (b->value() != aux->value()) { assignValue(aux, b->value(), PrgEdge::newEdge(*b, PrgEdge::Normal)); }
+			if (b->value() != value_free && !assignValue(aux, b->value(), PrgEdge::newEdge(*b, PrgEdge::Normal))) {
+				break;
+			}
 			aux->setInUpper(true);
 		}
 	}
