@@ -94,6 +94,7 @@ class FacadeTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testGenSolveWithLimit);
 	CPPUNIT_TEST(testCancelGenSolve);
 	CPPUNIT_TEST(testGenDtorCancelsSolve);
+	CPPUNIT_TEST(testGenStopFromHandler);
 #if WITH_THREADS
 	CPPUNIT_TEST(testGenSolveMt);
 #endif
@@ -814,7 +815,7 @@ public:
 		for (std::vector<std::string>::const_iterator it = keys.begin(), end = keys.end(); it != end; ++it) {
 			CPPUNIT_ASSERT(stats->type(stats->get(r, it->c_str())) == Potassco::Statistics_t::Value);
 		}
-		CPPUNIT_ASSERT_MESSAGE("unexpected number of statistics", keys.size() == 236);
+		CPPUNIT_ASSERT_MESSAGE("unexpected number of statistics", keys.size() == 237);
 	}
 	void testClingoStatsKeyIntegrity() {
 		Clasp::ClaspFacade libclasp;
@@ -1108,6 +1109,24 @@ public:
 		{ libclasp.startSolve(); }
 		CPPUNIT_ASSERT(!libclasp.solving() && !libclasp.result().exhausted());
 	}
+	void testGenStopFromHandler() {
+		ClaspConfig config;
+		ClaspFacade libclasp;
+		config.solve.numModels = 0;
+		lpAdd(libclasp.startAsp(config, true), "{x1}.");
+		libclasp.prepare();
+		struct Handler : EventHandler {
+			virtual bool onModel(const Solver&, const Model&) {
+				return false;
+			}
+		} h;
+		libclasp.ctx.setEventHandler(&h);
+		int mod = 0;
+		for (ClaspFacade::ModelGenerator g = libclasp.startSolve(); g.next(); ++mod) {
+			;
+		}
+		CPPUNIT_ASSERT(mod == 1);
+	}
 #if WITH_THREADS
 	void testGenSolveMt() {
 		ClaspConfig config;
@@ -1194,6 +1213,7 @@ class ClingoPropagatorTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testAddUnitClause);
 	CPPUNIT_TEST(testAddUnitClauseWithUndo);
 	CPPUNIT_TEST(testAddUnsatClause);
+	CPPUNIT_TEST(testAddEmptyClause);
 	CPPUNIT_TEST(testAttachToSolver);
 	CPPUNIT_TEST(testAddClauseOnModel);
 	CPPUNIT_TEST(testAddConflictOnModel);
@@ -1340,6 +1360,16 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, s.propagate());
 		CPPUNIT_ASSERT_MESSAGE("do not add sat constraint", ctx.numLearntShort() == 0);
 	}
+	void testAddEmptyClause() {
+		addVars(1);
+		tp->addWatch(prop.fire = negLit(v[1]));
+		prop.addToClause(negLit(0));
+		tp->addPost(*ctx.master());
+		ctx.endInit();
+		Solver& s = *ctx.master();
+		s.assume(negLit(v[1]));
+		CPPUNIT_ASSERT_EQUAL(false, s.propagate());
+	}
 	void testAttachToSolver() {
 		ClaspConfig config;
 		ClaspFacade libclasp;
