@@ -822,8 +822,8 @@ public:
 
 	void testDimacsBadVars() {
 		std::stringstream prg;
-		prg << "p cnf 2 1\n"
-		    << "3 4 0\n";
+		prg << "p cnf 1 1\n"
+		    << "1 2 0\n";
 		CPPUNIT_ASSERT_THROW(parse(api, prg), std::logic_error);
 	}
 
@@ -978,6 +978,8 @@ private:
 
 class OPBParserTest : public CppUnit::TestFixture {
   CPPUNIT_TEST_SUITE(OPBParserTest);
+	CPPUNIT_TEST(testBadVar);
+	CPPUNIT_TEST(testBadVarInGraph);
 	CPPUNIT_TEST(testWBO);
 	CPPUNIT_TEST(testNLC);
 	CPPUNIT_TEST(testNLCUnsorted);
@@ -991,6 +993,20 @@ public:
 	void setUp() {
 		api.startProgram(ctx);
 	}
+	void testBadVar() {
+		std::stringstream prg;
+		prg << "* #variable= 1 #constraint= 1\n+1 x2 >= 1;\n";
+		CPPUNIT_ASSERT_THROW(parse(api, prg), std::logic_error);
+	}
+	void testBadVarInGraph() {
+		std::stringstream prg;
+		prg << "* #variable= 1 #constraint= 1\n"
+		    << "* graph 2\n"
+		    << "* arc 1 1 2\n"
+		    << "* arc 2 2 1\n"
+		    << "* endgraph\n";
+		CPPUNIT_ASSERT_THROW(parse(api, prg, ParserOptions().enableAcycEdges()), std::logic_error);
+	}
 	void testWBO() {
 		std::stringstream prg;
 		prg << "* #variable= 1 #constraint= 2 #soft= 2 mincost= 2 maxcost= 3 sumcost= 5\n"
