@@ -4,16 +4,15 @@
 //  This is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
-//  (at your option) any later version. 
-// 
+//  (at your option) any later version.
+//
 //  This file is distributed in the hope that it will be useful,
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
 //  You should have received a copy of the GNU General Public License
-//  along with this file; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  along with this program. If not, see <http://www.gnu.org/licenses/>.
 //
 //
 // NOTE: ProgramOptions is inspired by Boost.Program_options
@@ -34,13 +33,13 @@
 namespace std { using ::size_t; }
 #endif
 
-namespace ProgramOptions { 
+namespace ProgramOptions {
 ///////////////////////////////////////////////////////////////////////////////
 // ValueMap
 ///////////////////////////////////////////////////////////////////////////////
-//! Type for storing anonymous values
+//! Type for storing anonymous values.
 /*!
- * Maps option names to their values
+ * Maps option names to their values.
  */
 class ValueMap {
 public:
@@ -81,7 +80,7 @@ private:
  * \see OptionGroup::addOptions()
  */
 template <class T>
-inline NotifiedValue<T>* store(ValueMap& map, typename detail::Parser<T>::type p = &DefaultParser<T>::parse) {
+inline NotifiedValue<T>* store(ValueMap& map, typename detail::Parser<T>::type p = &string_cast<T>) {
 	return notify<T>(&map, &ValueMap::add<T>, p);
 }
 
